@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2018-2020 Free Software Foundation, Inc.                   */
+/*  Copyright (C) 2018-2023 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -22,9 +22,11 @@
 #ifdef AX_STRCASECMP_HEADER
 #  include AX_STRCASECMP_HEADER
 #endif
-#include <sys/stat.h>
-#include <unistd.h>
-#include <getopt.h>
+#ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+#endif
+#include "my_stat.h"
+#include "my_getopt.h"
 #ifdef HAVE_VALGRIND_VALGRIND_H
 #  include <valgrind/valgrind.h>
 #endif
@@ -33,6 +35,7 @@
 #include "common.h"
 #include "bits.h"
 #include "suffix.inc"
+#include "my_getopt.h"
 #include "decode.h"
 #include "encode.h"
 #ifndef DISABLE_JSON
@@ -48,8 +51,9 @@ static int help (void);
 static int
 usage (void)
 {
-  printf ("\nUsage: dwgwrite [-v[0-9]] [-y] [--as rNNNN] [-I FMT] [-o DWGFILE] "
-          "INFILE\n");
+  printf (
+      "\nUsage: dwgwrite [-v[0-9]] [-y] [--as rNNNN] [-I FMT] [-o DWGFILE] "
+      "INFILE\n");
   return 1;
 }
 static int
@@ -68,16 +72,17 @@ help (void)
   printf ("  -v[0-9], --verbose [0-9]  verbosity\n");
   printf ("  --as rNNNN                save as version\n");
   printf ("           Valid versions:\n");
-  printf ("             r12, r14, r2000 (default)\n");
+  printf (
+      "             r1.1, r1.2, r1.3, r1.4, r2.0, r2.10, r2.21, r2.22, r2.4,"
+      "             r2.5, r2.6, r9, r10, r11, r13, r14, r2000 (default)\n");
   printf ("           Planned versions:\n");
-  printf ("             r9, r10, r11, r2004, r2007, r2010, r2013, r2018\n");
+  printf ("             r2004-r2021\n");
 #  ifndef DISABLE_JSON
   printf ("  -I fmt,  --format fmt     DXF, DXFB, JSON\n");
-#else
+#  else
   printf ("  -I fmt,  --format fmt     DXF, DXFB\n");
 #  endif
-  printf (
-      "           Planned input formats: GeoJSON, YAML, XML/OGR, GPX\n");
+  printf ("           Planned input formats: GeoJSON, YAML, XML/OGR, GPX\n");
   printf ("  -o dwgfile, --file        \n");
   printf ("  -y, --overwrite           overwrite existing files\n");
   printf ("           --help           display this help and exit\n");
@@ -87,15 +92,19 @@ help (void)
   printf ("  -v[0-9]     verbosity\n");
   printf ("  -a rNNNN    save as version\n");
   printf ("              Valid versions:\n");
-  printf ("                r12, r14, r2000 (default)\n");
+  printf (
+      "                r1.1, r1.2, r1.3, r1.4, r2.0, r2.10, r2.21, r2.22, "
+      "r2.4,"
+      "                r2.5, r2.6, r9, r10, r11, r13, r14, r2000 (default)\n");
   printf ("              Planned versions:\n");
-  printf ("                r9, r10, r11, r2004, r2007, r2010, r2013, r2018\n");
+  printf ("                r2004-r2021\n");
 #  ifndef DISABLE_JSON
   printf ("  -I fmt      fmt: DXF, DXFB, JSON\n");
 #  else
   printf ("  -I fmt      fmt: DXF, DXFB\n");
-#endif
-  printf ("              Planned input formats: GeoJSON, YAML, XML/OGR, GPX\n");
+#  endif
+  printf (
+      "              Planned input formats: GeoJSON, YAML, XML/OGR, GPX\n");
   printf ("  -o dwgfile\n");
   printf ("  -y          overwrite existing files\n");
   printf ("  -h          display this help and exit\n");
@@ -108,53 +117,63 @@ help (void)
 }
 
 #ifdef __AFL_COMPILER
-__AFL_FUZZ_INIT();
-int main (int argc, char *argv[])
+__AFL_FUZZ_INIT ();
+// fastest mode via shared mem (crashes still)
+#  define AFL_SHARED_MEM
+
+int
+main (int argc, char *argv[])
 {
   Dwg_Data dwg;
   Bit_Chain dat = { NULL, 0, 0, 0, 0 };
   Bit_Chain out_dat = { NULL, 0, 0, 0, 0 };
   FILE *fp;
 
-  __AFL_INIT();
+  __AFL_INIT ();
   dat.chain = NULL;
   dat.version = R_2000;
   printf ("Fuzzing in_json + encode from shared memory\n");
 
-  while (__AFL_LOOP(10000)) { // llvm_mode persistent, non-forking mode
-#if 1 // fastest mode via shared mem (crashes still)
-    dat.chain = __AFL_FUZZ_TESTCASE_BUF;
-    dat.size = __AFL_FUZZ_TESTCASE_LEN;
-#elif 1 // still 1000x faster than the old file-forking fuzzer.
-    /* from stdin: */
-    dat.size = 0;
-    dat_read_stream (&dat, stdin);
-#else
-    /* else from file */
-    stat (argv[1], &attrib);
-    fp = fopen (argv[1], "rb");
-    if (!fp)
-      return 0;
-    dat.size = attrib.st_size;
-    dat_read_file (&dat, fp, argv[1]);
-    fclose (fp);
-#endif
-    if (dat.size < 100) continue;  // useful minimum input length
+#  ifdef AFL_SHARED_MEM
+  dat.chain = __AFL_FUZZ_TESTCASE_BUF;
+#  endif
+  while (__AFL_LOOP (10000))
+    { // llvm_mode persistent, non-forking mode
+#  ifdef AFL_SHARED_MEM
+      dat.size = __AFL_FUZZ_TESTCASE_LEN;
+#  elif 1 // still 1000x faster than the old file-forking fuzzer.
+      /* from stdin: */
+      dat.size = 0;
+      dat_read_stream (&dat, stdin);
+#  else
+      /* else from file */
+      stat (argv[1], &attrib);
+      fp = fopen (argv[1], "rb");
+      if (!fp)
+        return 0;
+      dat.size = attrib.st_size;
+      dat_read_file (&dat, fp, argv[1]);
+      fclose (fp);
+#  endif
+      if (dat.size < 100)
+        continue; // useful minimum input length
 
-    if (dwg_read_json (&dat, &dwg) <= DWG_ERR_CRITICAL) {
-      memset (&out_dat, 0, sizeof (out_dat));
-      bit_chain_set_version (&out_dat, &dat);
-      out_dat.version = R_2000;      
-      if (dwg_encode (&dwg, &out_dat) >= DWG_ERR_CRITICAL)
+      if (dwg_read_json (&dat, &dwg) <= DWG_ERR_CRITICAL)
+        {
+          memset (&out_dat, 0, sizeof (out_dat));
+          bit_chain_set_version (&out_dat, &dat);
+          out_dat.version = R_2000;
+          out_dat.codepage = dwg.header.codepage;
+          if (dwg_encode (&dwg, &out_dat) >= DWG_ERR_CRITICAL)
+            exit (0);
+          free (out_dat.chain);
+        }
+      else
         exit (0);
-      free (out_dat.chain);
     }
-    else
-      exit (0);
-  }
   dwg_free (&dwg);
 }
-#define main orig_main
+#  define main orig_main
 int orig_main (int argc, char *argv[]);
 #endif
 
@@ -169,7 +188,7 @@ main (int argc, char *argv[])
   char *outfile = NULL;
   Bit_Chain dat = { 0 };
   const char *version = NULL;
-  Dwg_Version_Type dwg_version = R_2000;
+  Dwg_Version_Type dwg_version = R_INVALID;
   int c;
   int force_free = 0;
   int free_outfile = 0;
@@ -177,25 +196,22 @@ main (int argc, char *argv[])
   int option_index = 0;
   static struct option long_options[]
       = { { "verbose", 1, &opts, 1 }, // optional
-          { "format", 1, 0, 'I' },    { "file", 1, 0, 'o' },
-          { "as", 1, 0, 'a' },        { "help", 0, 0, 0 },
-          { "overwrite", 0, 0, 'y' }, { "version", 0, 0, 0 },
-          { "force-free", 0, 0, 0 },
-          { NULL, 0, NULL, 0 } };
+          { "format", 1, NULL, 'I' },    { "file", 1, NULL, 'o' },
+          { "as", 1, NULL, 'a' },        { "help", 0, NULL, 'h' },
+          { "overwrite", 0, NULL, 'y' }, { "version", 0, NULL, 0 },
+          { "force-free", 0, NULL, 0 },  { NULL, 0, NULL, 0 } };
 #endif
 
   if (argc < 2)
     return usage ();
 
-  while
-#ifdef HAVE_GETOPT_LONG
-      ((c
-        = getopt_long (argc, argv, "ya:v::I:o:h", long_options, &option_index))
-       != -1)
-#else
-      ((c = getopt (argc, argv, "ya:v::I:o:hi")) != -1)
-#endif
+  while (1)
     {
+#ifdef HAVE_GETOPT_LONG
+      c = getopt_long (argc, argv, "ya:v::I:o:h", long_options, &option_index);
+#else
+      c = getopt (argc, argv, "ya:v::I:o:hi");
+#endif
       if (c == -1)
         break;
       switch (c)
@@ -228,8 +244,6 @@ main (int argc, char *argv[])
             }
           if (!strcmp (long_options[option_index].name, "version"))
             return opt_version ();
-          if (!strcmp (long_options[option_index].name, "help"))
-            return help ();
           if (!strcmp (long_options[option_index].name, "force-free"))
             force_free = 1;
           break;
@@ -298,7 +312,7 @@ main (int argc, char *argv[])
             fmt = (char *)"json";
           else
 #  endif
-          if (strstr (infile, ".dxfb") || strstr (infile, ".DXFB"))
+              if (strstr (infile, ".dxfb") || strstr (infile, ".DXFB"))
             fmt = (char *)"dxfb";
           else if (strstr (infile, ".dxf") || strstr (infile, ".DXF"))
             fmt = (char *)"dxf";
@@ -307,7 +321,7 @@ main (int argc, char *argv[])
             fprintf (stderr, "Unknown input format for '%s'\n", infile);
         }
     }
-  
+
   // allow stdin, but require -I|--format then
   memset (&dwg, 0, sizeof (Dwg_Data));
   dat.opts = dwg.opts = opts;
@@ -348,14 +362,15 @@ main (int argc, char *argv[])
     }
   else
 #  endif
-  if ((fmt && !strcasecmp (fmt, "dxfb"))
-           || (infile && !strcasecmp (infile, ".dxfb")))
+      if ((fmt && !strcasecmp (fmt, "dxfb"))
+          || (infile && !strcasecmp (infile, ".dxfb")))
     {
       if (opts > 1)
         {
           fprintf (stderr, "Reading Binary DXF file %s\n",
-                 infile ? infile : "from stdin");
-          fprintf (stderr, "Warning: still highly experimental and untested.\n");
+                   infile ? infile : "from stdin");
+          fprintf (stderr,
+                   "Warning: still highly experimental and untested.\n");
         }
       if (infile)
         error = dxf_read_file (infile, &dwg); // ascii or binary
@@ -396,13 +411,18 @@ main (int argc, char *argv[])
   if (error >= DWG_ERR_CRITICAL)
     goto free;
 
+  if (dwg_version == R_INVALID)
+    {
+      dwg_version = dwg.header.from_version;
+      if (dwg_version >= R_2004)
+        dwg_version = R_2000;
+      else if (dwg_version < R_1_4)
+        dwg_version = R_1_4;
+    }
   if (dwg.header.from_version == R_INVALID)
     fprintf (stderr, "Unknown DWG header.from_version");
-  // FIXME: for now only R_13 - R_2000. later remove this line.
-  if (dwg.header.from_version < R_13 || dwg.header.from_version >= R_2004)
-    dat.version = dwg.header.version = dwg_version;
-  else
-    dat.version = dwg.header.version = dwg.header.from_version;
+  // FIXME: for now only r1.4 - R_2000. later remove this line.
+  dat.version = dwg.header.version = dwg_version;
 
   if (!outfile)
     {
@@ -438,11 +458,11 @@ main (int argc, char *argv[])
               }
             else if ( // for fuzzing mainly
 #ifdef _WIN32
-                     strEQc (outfile, "NUL")
+                strEQc (outfile, "NUL")
 #else
-                     strEQc (outfile, "/dev/null")
+                strEQc (outfile, "/dev/null")
 #endif
-                     )
+            )
               {
                 dwg.opts |= opts;
                 error = dwg_write_file (outfile, &dwg);
@@ -462,10 +482,10 @@ main (int argc, char *argv[])
       }
   }
 
-  free:
+free:
 #if defined __SANITIZE_ADDRESS__ || __has_feature(address_sanitizer)
   {
-    char *asanenv = getenv("ASAN_OPTIONS");
+    char *asanenv = getenv ("ASAN_OPTIONS");
     if (!asanenv)
       force_free = 1;
     // detect_leaks is enabled by default. see if it's turned off
@@ -474,12 +494,11 @@ main (int argc, char *argv[])
   }
 #endif
   // forget about leaks. really huge DWG's need endlessly here.
-  if ((dwg.header.version && dwg.num_objects < 1000)
-      || force_free
+  if ((dwg.header.version && dwg.num_objects < 1000) || force_free
 #ifdef HAVE_VALGRIND_VALGRIND_H
       || (RUNNING_ON_VALGRIND)
 #endif
-      )
+  )
     {
       dwg_free (&dwg);
     }
