@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2019 Free Software Foundation, Inc.                        */
+/*  Copyright (C) 2019,2023 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -23,26 +23,29 @@
 #include "common.h"
 #include "escape.h"
 
-char * ATTRIBUTE_MALLOC
-htmlescape (const char *restrict src, const int cp)
+char *ATTRIBUTE_MALLOC
+htmlescape (const char *restrict src, const Dwg_Codepage cp)
 {
-  int len;
+  size_t len;
   char *dest, *d, *end;
   unsigned char *s;
   if (!src)
     return NULL;
   len = strlen (src) + 10;
-  d = calloc (len, 1);
+  d = (char *)calloc (len, 1);
   s = (unsigned char *)src;
   dest = d;
   end = dest + len;
   while (*s)
     {
-      const int off = d - dest;
       if (end - d <= 8)
         {
+          const int off = d - dest;
+          char *newdest = (char *)realloc (dest, len + 10);
+          if (!newdest)
+            return NULL;
+          dest = newdest;
           len += 10;
-          dest = realloc (dest, len);
           d = dest + off;
           *d = 0;
           end = dest + len;
@@ -82,17 +85,23 @@ htmlescape (const char *restrict src, const int cp)
           d += 6;
           break;
         default:
-          if (*s >= 127) // maybe encodings, no utf8 (see htmlwescape)
-            {
-              sprintf (d, "&#x%X;", *s); // 4 + 4
-              d += strlen (d);
-              *d = 0;
-            }
-          else if (*s >= 20)
-            {
-              *d++ = *s;
-              *d = 0;
-            }
+          {
+            uint16_t cc = *s;
+            wchar_t wc;
+            if (dwg_codepage_is_twobyte (cp, *s))
+              cc = cc << 8 | *++s;
+            wc = dwg_codepage_uwc (cp, cc);
+            if (wc > 127 || wc < 0x20)
+              {
+                sprintf (d, "&#x%X;", (unsigned)wc); // 4 + 4
+                d += strlen (d);
+              }
+            else
+              {
+                *d++ = *s;
+              }
+            *d = 0;
+          }
         }
       s++;
     }
@@ -100,7 +109,7 @@ htmlescape (const char *restrict src, const int cp)
   return dest;
 }
 
-char * ATTRIBUTE_MALLOC
+char *ATTRIBUTE_MALLOC
 htmlwescape (BITCODE_TU wstr)
 {
   int len = 0;
@@ -113,36 +122,63 @@ htmlwescape (BITCODE_TU wstr)
   while ((c = *tmp++))
     len++;
   len += 16;
-  d = dest = calloc (len, 1);
+  d = dest = (char *)calloc (len, 1);
 
   while (*wstr)
     {
       const int off = d - dest;
       if (off >= len - 8)
         {
+          char *newdest = (char *)realloc (dest, len + 16);
+          if (!newdest)
+            return NULL;
+          dest = newdest;
           len += 16;
-          dest = realloc (dest, len);
           d = dest + off;
           *d = 0;
         }
       switch (*wstr)
         {
-        case 34: strcat (d, "&quot;"); d += 6; break;
-        case 39: strcat (d, "&#39;"); d += 5; break;
-        case 38: strcat (d, "&amp;"); d += 5; break;
-        case 60: strcat (d, "&lt;"); d += 4; break;
-        case 62: strcat (d, "&gt;"); d += 4; break;
-        case 96: strcat (d, "&#96;"); d += 5; break;
-        case 123: strcat (d, "&#123;"); d += 6; break;
-        case 125: strcat (d, "&#125;"); d += 6; break;
+        case 34:
+          strcat (d, "&quot;");
+          d += 6;
+          break;
+        case 39:
+          strcat (d, "&#39;");
+          d += 5;
+          break;
+        case 38:
+          strcat (d, "&amp;");
+          d += 5;
+          break;
+        case 60:
+          strcat (d, "&lt;");
+          d += 4;
+          break;
+        case 62:
+          strcat (d, "&gt;");
+          d += 4;
+          break;
+        case 96:
+          strcat (d, "&#96;");
+          d += 5;
+          break;
+        case 123:
+          strcat (d, "&#123;");
+          d += 6;
+          break;
+        case 125:
+          strcat (d, "&#125;");
+          d += 6;
+          break;
         default:
-          if (*wstr >= 127) // utf8 encodings
+          if (*wstr >= 127 || *wstr < 20) // utf8 encodings
             {
               sprintf (d, "&#x%X;", *wstr);
               d += strlen (d);
               *d = 0;
             }
-          else if (*wstr >= 20)
+          else
             {
               *d++ = *wstr;
               *d = 0;
