@@ -5,7 +5,7 @@ void
 api_process (dwg_object *obj)
 {
   int error, isnew;
-  BLOCKACTION_WITHBASEPT_fields(3);
+  BLOCKACTION_WITHBASEPT_fields (3);
   BITCODE_BL i;
 
   Dwg_Version_Type dwg_version = obj->parent->header.version;
@@ -21,16 +21,20 @@ api_process (dwg_object *obj)
   CHK_ENTITY_3RD (_obj, BLOCKROTATEACTION, offset);
   for (i = 0; i < 2; i++)
     {
-      CHK_SUBCLASS_TYPE     (_obj->conn_pts[i], BLOCKACTION_connectionpts, code, BL);
-      CHK_SUBCLASS_UTF8TEXT (_obj->conn_pts[i], BLOCKACTION_connectionpts, name);
+      CHK_SUBCLASS_TYPE (_obj->conn_pts[i], BLOCKACTION_connectionpts, code,
+                         BL);
+      CHK_SUBCLASS_UTF8TEXT (_obj->conn_pts[i], BLOCKACTION_connectionpts,
+                             name);
     }
   CHK_ENTITY_TYPE (_obj, BLOCKROTATEACTION, dependent, B);
   CHK_ENTITY_3RD (_obj, BLOCKROTATEACTION, base_pt);
-  //CHK_ENTITY_3RD (_obj, BLOCKROTATEACTION, stretch_pt);
-  // AcDbBlockRotateAction
+  // CHK_ENTITY_3RD (_obj, BLOCKROTATEACTION, stretch_pt);
+  //  AcDbBlockRotateAction
   for (i = 2; i < 3; i++)
     {
-      CHK_SUBCLASS_TYPE     (_obj->conn_pts[i], BLOCKACTION_connectionpts, code, BL);
-      CHK_SUBCLASS_UTF8TEXT (_obj->conn_pts[i], BLOCKACTION_connectionpts, name);
+      CHK_SUBCLASS_TYPE (_obj->conn_pts[i], BLOCKACTION_connectionpts, code,
+                         BL);
+      CHK_SUBCLASS_UTF8TEXT (_obj->conn_pts[i], BLOCKACTION_connectionpts,
+                             name);
     }
 }
